#ifndef bwtScreenRegionHH
#define bqtScreenRegionHH

/* included only from cons.hh */

extern class Screen
{
public:
    struct xycoord
    {
        unsigned x, y;
        unsigned area() const { return x*y; }
    };
    class ScreenRegion
    {
    public:
        ScreenRegion() : owner(NULL) { }
        ScreenRegion(Screen *o, xycoord asize, xycoord aorig, bool temp=false);
        ~ScreenRegion();

        inline void setscrolling(bool value=true) { scrolling = value; }
        inline void settemporary(bool value=true) { temporary = value; }
        
        unsigned printf(const char *fmt, va_list ap);
        unsigned printf(const char *fmt ...) PRNATTR(1);
        void putchar(char c, unsigned count);
        void putchar(char c);
        void puts(const string &s, unsigned count=1);
        void setattr(BYTE newattr) { attr=newattr; }
        inline void gotox(unsigned x)
        {
            pos.x = x;
        }
        inline void gotoxy(unsigned x, unsigned y)
        {
            pos.x = x;
            pos.y = y;
        }
        inline unsigned wherex() const { return pos.x; }
        inline unsigned wherey() const { return pos.y; }
        void flush();
        
        inline bool operator< (int) const { return owner==NULL; }
        inline unsigned height() const { return size.y; }
        inline unsigned width() const { return size.x; }
        inline unsigned ysize() const { return size.y; }
        inline unsigned xsize() const { return size.x; }
        
    private:
        Screen *const owner;
        bool scrolling;
        bool temporary;
        BYTE attr;
        
        xycoord orig, size, pos;
        BYTE *framebuffer;
    };
public:
    static const unsigned nsize = static_cast<unsigned>(-1);
    
    Screen();
    ~Screen();
    
    void GetScreenGeometry();
    
    inline unsigned height() const { return LINES; }
    inline unsigned width() const { return COLS; }
    inline unsigned ysize() const { return LINES; }
    inline unsigned xsize() const { return COLS; }
    
    ScreenRegion AllocateBox(unsigned ysize=nsize, unsigned xsize=nsize, bool temporary=false);
    inline ScreenRegion AllocateLine(unsigned xsiz=nsize, bool temporary=false)
    {
        return AllocateBox(1, xsiz, temporary);
    }
    inline ScreenRegion AllocateLine(bool temporary)
    {
        return AllocateBox(1, nsize, temporary);
    }
    
    void UnderlineCursor();
    void BlockCursor();
    
private:
    friend class ScreenRegion;
    void allocate(const xycoord &orig, const xycoord &size);
    void deallocate(const xycoord &orig, const xycoord &size, bool really = true);
    void Write(unsigned x,unsigned y, BYTE attr, char c);
    void Write(unsigned x,unsigned y);
    void Locate(const unsigned x, const unsigned y);
    void Flush();
    
    unsigned xpos, ypos;
    BYTE textattr;
    unsigned LINES, COLS;
    
    class UsedMap *Allocated;
public:
    void Color(BYTE attr);
} Display;

#endif
